/// <reference path="../Scripts/typings/jquery/jquery.d.ts"/>
/// <reference path="../Scripts/typings/jqueryui/jqueryui.d.ts"/>

// Application model describes all windows used in the application
interface IApplicationModel {
    // Collection of window models
    windows?: IWindowModel[];
}

// Describes a control
interface IControlModel {
    // Control type
    type: string;
    // Control name
    name?: string;
    // Control css class
    cssClass?: string;
    // Specifies whether control is hidden
    isHidden?: boolean;
}

// Describes a databound control
interface IDataBoundControlModel extends IControlModel {
    // The property of the data object that the control binds to
    bindsTo?: string;
}

// Describes a control that can contain other controls
interface IContainerControlModel extends IControlModel {
    // Collection of child controls
    controls?: IControlModel[];
    // The property of the data object that will be used as a binding context.
    // When not set, the data object itself is used as a context.
    bindingContext?: string;
}

// Describes an application window
interface IWindowModel extends IContainerControlModel {
    // The window title
    title?: string;
    // The x coordinate of the left top corner
    left?: number;
    // The y coordinate of the left top corner
    top?: number;
    // The width
    width?: number;
    // The height
    height?: number;
}

// Describes a group box
interface IGroupBoxModel extends IContainerControlModel {
    // The label
    label?: string;
}

// Describes a text box
interface ITextBoxModel extends IDataBoundControlModel {
    // The label
    label?: string;
}

// Describes a text area
interface ITextAreaModel extends IDataBoundControlModel {
    // The label
    label?: string;
    // Specifies the visible width of a text area
    columns?: number;
    // Specifies the visible number of lines in a text area
    rows?: number;
}

// Describes a check box
interface ICheckBoxModel extends IDataBoundControlModel {
    // The label
    label?: string;
}

// Describes a combo box
interface IComboBoxModel extends IDataBoundControlModel {
    // The label
    label?: string;
    // The name of the function to get the possible values.
    getOptionsFunction?: string;
    // The name of the property that is used as a option id.
    optionIdProperty?: string;
    // The name of the property that is used as a option value.
    optionValueProperty?: string;
}

// Describes a button
interface IButtonModel extends IControlModel {
    // The button text
    text?: string;
    // Specifies whether the button is a dialog button
    isDialogButton?: boolean;
    // Specifies the dialog result.
    dialogResult?: string;
}

// Describes a grid view
interface IGridViewModel extends IDataBoundControlModel {
    // The label
    label?: string;
    // Columns
    columns?: IGridViewColumnModel[];
    // Specifies whether user can add rows
    canUserAddRows?: boolean;
    // The path to the function to create a new row
    createNewRowValueFunction?: string;
}

// Describes a grid view column
interface IGridViewColumnModel extends IDataBoundControlModel {
    // The label
    label?: string;
    // The width
    width?: number;
}

// Describes a grid view combobox column
interface IGridViewComboBoxColumnModel extends IGridViewColumnModel {
    // The name of the function to get the possible values
    getOptionsFunction?: string;
    // The name of the property that is used as a option id.
    optionIdProperty?: string;
    // The name of the property that is used as a option value.
    optionValueProperty?: string;
}

// Describes a tree view
interface ITreeViewModel extends IDataBoundControlModel {
    // The label
    label?: string;
    // A model of a single node
    nodeModel: ITreeViewNodeModel; // TODO: should be optional?
}

// Describes a tree view node
interface ITreeViewNodeModel extends IContainerControlModel, IDataBoundControlModel {
}

// Describes a static text
interface IStaticTextModel extends IDataBoundControlModel {
    // Text
    text?: string;
}

// Describes an image
interface IImageModel extends IDataBoundControlModel {
    // The source of the image
    source?: string;
    // The alternate text for an image
    alternateText?: string;
    // The width
    width?: number;
    // The height
    height?: number;
}

// Describes a generic object selector
interface IObjectSelectorBaseModel extends IControlModel {
    // The name of the function to get the object
    getObjectFunction?: string;
    // The name of the property that is used as an object value.
    objectValueProperty?: string;
}

// Describes an object selector
interface IObjectSelectorModel extends IObjectSelectorBaseModel, IDataBoundControlModel {
    // The label
    label?: string;
}

// Describes a grid view object selector column
interface IGridViewObjectSelectorColumnModel extends IObjectSelectorBaseModel, IGridViewColumnModel {
}

// Describes a pager
interface IPagerModel extends IDataBoundControlModel {
    // The name of the property that has a number of the current page.
    pageNoProperty?: string;
    // The name of the property that has a total number of pages.
    pagesTotalProperty?: string;
}

// *****************************************************************************************
// *                             Controls                                                  *
// *****************************************************************************************

// The control
interface IControl {
    // The control model
    model: IControlModel;
    // The control id
    id: string;
    // The control type
    type: string;
    // The control name
    name: string;
    // The parent control
    parent: IControl;
    // The window
    window: IWindow;
    // The function that returns jQuery function to access rendered HTML element
    getElement(): JQuery;
    // The function that returns the html rendered from the control
    _toHtml(): string;
}

// The databound control
interface IDataBoundControl extends IControl {
    // The object that is used as a binding context.
    bindingContext: any;
    // Binds the control to the data object
    bind(dataObject: any): void;
}

// The control that can contain other controls
interface IContainerControl extends IDataBoundControl {
    // Collection of child controls
    controls: IControl[];
}

// The window
interface IWindow extends IContainerControl {
    // All child controls by id
    _allControls: any;
    // The x coordinate of the left top corner
    left: number;
    // The y coordinate of the left top corner
    top: number;
    // The width
    width: number;
    // The height
    height: number;
    // The width
    titleWidth: number;
    // The height
    titleHeight: number;
    // The width
    clientAreaWidth: number;
    // The height
    clientAreaHeight: number;
    // Shows the window.
    show(): void;
    // Hides the window.
    close(dialogResult?: string): void;
    // Shows the window as a modal dialog.
    showModal(): void;
    // Docks the window into the div.
    dock(divSelector: JQuery): void;
    // If window is docked, undocks the window from the div. 
    // If the window is not docked, does nothing.
    undock(): void;
    // Is triggered before the window is closed.
    closing(handler: (target: any, dialogResult: string) => void): void;
    // Is triggered after the window is closed.
    closed(handler: (target: any, dialogResult: string) => void): void;
}

// The group box
interface IGroupBox extends IContainerControl {
}

// The textbox
interface ITextBox extends IDataBoundControl {
}

// The textarea
interface ITextArea extends IDataBoundControl {
}

// The checkbox
interface ICheckBox extends IDataBoundControl {
}

// The combobox
interface IComboBox extends IDataBoundControl {
    // Combobox options
    options: any[];
}

// The button
interface IButton extends IControl {
}

// The grid view
interface IGridView extends IDataBoundControl {
    // Columns
    columns: IGridViewColumn[];
    // Roes
    rows: IGridViewRow[];
    // Current row
    currentRow: IGridViewRow;
    // Creates a new row
    createNewRowValue(): () => any;
    // Is triggered when the row is selected.
    rowSelected(handler: (target: any, row: IGridViewRow) => void): void;
}

// The grid view column
interface IGridViewColumn extends IControl {
}

// The grid view combobox column
interface IGridViewComboBoxColumn extends IGridViewColumn {
    // Combobox options
    options: any;
}

// The grid view row
interface IGridViewRow extends IDataBoundControl {
    // Row cells
    cells: IGridViewCell[];
    // Specifies whether the row is a sentinel row
    isSentinel: boolean;
}

// The grid view cell
interface IGridViewCell extends IDataBoundControl {
    // The row the cell belongs to
    row: IGridViewRow;
    // The cell initial value
    value: string;
}

// The grid view combobox cell
interface IGridViewComboBoxCell extends IGridViewCell {
    // Combobox options
    options: any;
}

// The tree view
interface ITreeView extends IDataBoundControl {
    // The root nodes of the tree view
    roots: ITreeViewNode[];
    // Currently selected node
    currentNode: ITreeViewNode;
    // Is triggered when the node is selected.
    nodeSelected(handler: (target: any, node: ITreeViewNode) => void): void;
    // Is triggered when the node is expanded.
    nodeExpanded(handler: (target: any, node: ITreeViewNode) => void): void;
}

// The tree view node
interface ITreeViewNode extends IContainerControl {
    // The children nodes of the current node
    children: ITreeViewNode[];
    // The tree view
    treeView: ITreeView;
}

// RedUI object
interface IRedUI {
    // Application model
    model: IApplicationModel;
    // Application windows
    windows: any;
    // Creates a new window. 
    // windowName is used to find the window model in the application model, must match window model name property.
    createNewWindow: { (windowName: string): IWindow; };
    // Options bag. Used to access functions returning options for comboboxes.
    optionsBag: any;
    // Object sources. Used to access functions returning objects for object selectors.
    objectSources: any;
}

// The static text
interface IStaticText extends IDataBoundControl {
}

// The image
interface IImage extends IDataBoundControl {
}

// The object selector
interface IObjectSelector extends IDataBoundControl {
}

// The pager
interface IPager extends IDataBoundControl {
}

// The pager info
interface IPagerInfo extends IDataBoundControl {
    // Current page number
    pageNo: number;
    // Total number of pages
    pagesTotal: number;
}